#region usings
using System;
using System.Runtime.InteropServices;
using System.ComponentModel.Composition;
using System.Collections;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "MultiHWND",
				Category = "Windows",
				Help = "Gets the handle of a window with the given title. Specializes in storing and updating lists of multiple windows with the same title.",
				Author = "Grandchild",
				Tags = "HWND, Windows")]
	#endregion PluginInfo
	public class WindowsMultiHWNDNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Title")]
		ISpread<string> FTitle;
		
		[Input("Remove Invalid Handles", DefaultValue = 0)]
		ISpread<bool> FRemoveInvalid;
		
		[Input("Reset", DefaultValue = 0, IsBang = true)]
		ISpread<bool> FClear;
		
		[Output("Handle List")]
		ISpread<ISpread<int>> FOutput;
		
		ArrayList titles;
		#endregion fields & pins
		
		//[DllImport("%windir%\\System32\\user32.dll")]
		[DllImport("C:\\Windows\\System32\\user32.dll")]
		public static extern IntPtr FindWindow(String sClassName, String sAppName);
		
		[DllImport("C:\\Windows\\System32\\user32.dll")]
		public static extern bool IsWindow(IntPtr hwnd);
		
		WindowsMultiHWNDNode()
		{
			titles = new ArrayList();
			titles.Add("");
		}
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			SpreadMax = FTitle.SliceCount;
			FOutput.SliceCount = SpreadMax;
						
			// add or remove items from titles-list as neccessary
			for(int i=titles.Count; i<SpreadMax; i++)	titles.Add(FTitle[i]);
			while(SpreadMax<titles.Count)				titles.RemoveAt(titles.Count-1);
			
			for(int i=0; i<FTitle.SliceCount; i++)
			{
				// if clear was set or the window title has changed since last frame, clear the handle list
				if(FClear[i%SpreadMax] ||
					titles[i]!=FTitle[i])
					FOutput[i].SliceCount=0;
				
				// get the handle for the currently inserted title(s)
				int curWindowHandle = FindWindow(null,FTitle[i]).ToInt32();
				
				// if there are no handles yet, simply add the current one
				if(FOutput[i].SliceCount==0) {
					FOutput[i].Add(curWindowHandle);
				}
				
				// now look if the current handle is already in the output list
				bool match = false;
				bool foundInvalid = false;
				int firstInvalid = -1;
				for(int k=0; k<FOutput[i].SliceCount; k++)
				{
					// if we encounter an invalid handle we replace it with -1
					if(!IsWindow(new IntPtr(FOutput[i][k])))
					{
						// if "remove invalid" is set, simply remove it
						if(FRemoveInvalid[i%SpreadMax])
						{
							FOutput[i].RemoveAt(k);
						}
						else
						{
							FOutput[i][k] = -1;
							
							
							// mark and remember the first invalid handle, if any
							if(!foundInvalid)
								firstInvalid = k;
							foundInvalid = true;
						}
					}
					
					// compare and if found set the match flag
					if(FOutput[i][k]==curWindowHandle)
						match = true;
				}
				
				// if we have a new handle (and it's a valid one)...
				if(!match && (curWindowHandle>0))
				{
					// ... either put it where a handle has become invalid
					if(foundInvalid)
						FOutput[i][firstInvalid] = curWindowHandle;
					// or add it to the end of the list
					else
						FOutput[i].Add(curWindowHandle);
				}
				
				// update list item
				titles[i] = FTitle[i];
			}
		}
	}
}